import requests

try:
    from word import new as new_word
except:
    from .word import new as new_word


class LookupException(Exception):
    pass


URL = 'https://sozluk.gov.tr/gts?ara={}'
USER_AGENT = (
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_3) '
    'AppleWebKit/537.36 (KHTML, like Gecko) '
    'Chrome/80.0.3987.87 Safari/537.36'
)
HEADERS = {
    'Host': 'sozluk.gov.tr',
    'User-Agent': USER_AGENT
}


def lookup(search):
    if search is None:
        return None

    try:
        response = requests.get(URL.format(search), headers=HEADERS)
        result = response.json()

        if 'error' in result:
            return None

        return new_word(result)
    except Exception as e:
        raise LookupException('lookup error') from e



from collections import namedtuple

Variation = namedtuple(
    'Variation', 'name origin meanings compound_words proverbs'
)
Meaning = namedtuple(
    'Meaning', 'text props samples'
)
Sample = namedtuple(
    'Sample', 'text authors'
)


class Word:
    __variations = []

    def __init__(self, words):
        for variation in words:
            self.__parse_variations(variation)

    def __parse_variations(self, variation):
        name                = variation['madde']
        origin              = variation['lisan']
        meanings            = self.__parse_meanings(variation)
        compound_words      = self.__parse_compound_words(variation)
        proverbs            = self.__parse_proverbs(variation)

        self.__variations.append(Variation(
            name, origin, meanings, compound_words, proverbs,
        ))

    def __parse_meanings(self, variation):
        meanings    = []
        _meanings   = variation.get('anlamlarListe', [])
        for m in _meanings:
            text        = m['anlam']
            props       = self.__parse_props(m)
            samples     = self.__parse_samples(m),

            if len(samples) > 0:
                samples = samples[0]

            meanings.append(Meaning(text, props, samples))

        return meanings

    def __parse_props(self, meaning):
        return list(map(
            lambda p: p['tam_adi'],
            meaning.get('ozelliklerListe', [])
        ))

    def __parse_samples(self, meaning):
        samples     = []
        _samples    = meaning.get('orneklerListe', [])

        for s in _samples:
            text    = s.get('ornek')
            authors = list(map(
                lambda a: a.get('tam_adi'),
                s.get('yazar', []),
            ))

            samples.append(Sample(text, authors))

        return samples

    def __parse_compound_words(self, variation):
        compound_words = variation.get('birlesikler', '')

        if compound_words is None: return []

        return list(map(
            lambda cw: cw.strip(),
            compound_words.split(','),
        ))

    def __parse_proverbs(self, variation):
        proverbs = variation.get('atasozu', [])

        return list(map(
            lambda p: p['madde'],
            proverbs,
        ))

    def get_variations(self):
        return self.__variations


def new(word):
    return Word(word)



import sys

from termcolor import colored

try:
    from dict import lookup, LookupException
except:
    from .dict import lookup, LookupException

def main():
    search = ' '.join(sys.argv[1:])
    if not search:
        print("Lütfen en az bir harf yazınız.")
        sys.exit(0)

    word = lookup(search)

    if word is not None:
        variations = word.get_variations()

        def colorize_variations():
            text = ''
            for i, v in enumerate(variations, start=1):
                text += colored(v.name, 'yellow', attrs=['bold', 'underline'])
                if len(variations) > 2:
                    text += ' {}'.format(colored(f'[{i}]', attrs=['dark']))
                if v.origin:
                    text += ' - ' + colored(v.origin, 'yellow', attrs=['dark'])

                text += '\n\n'
                for m in v.meanings:
                    text += '• ' + m.text + ' '

                    if m.props:
                        text += colored(
                            ', '.join(m.props),
                            'yellow', attrs=['dark'],
                        )

                    text += '\n'
                    for s in m.samples:
                        text += f"""  {colored('Örnek', 'magenta')}  """
                        text += f"""  {colored(s.text, 'green')}"""
                        if s.authors:
                            text += '\n'
                            text += f"""  {colored('Yazarlar', 'magenta')} """
                            text += colored(', '.join(s.authors), 'blue')
                        text += '\n'
                    else:
                        text += '\n'

                if v.compound_words:
                    text += colored('Birleşik Kelimeler', 'cyan') + '\n'
                    text += '\n'.join(['• ' + c for c in v.compound_words])
                    text += '\n\n'

                if v.proverbs:
                    text += colored(
                        'Atasözleri, Deyimler veya Birleşik Fiiller', 'cyan'
                    )
                    text += '\n'
                    text += '\n'.join(['• ' + p for p in v.proverbs])
                    text += '\n'

            return text

        out = colorize_variations()

        print(out.rstrip('\n'))


if __name__ == "__main__":
    main()



import re
from setuptools import setup


version = re.search(
    '^__version__\s*=\s*"(.*)"',
    open('lugat/__version__.py').read(),
    re.M
).group(1)


with open("README.rst", "rb") as f:
    long_descr = f.read().decode("utf-8")


setup(
    name="lugat",
    packages=["lugat"],
    entry_points={
        "console_scripts": ['lugat = lugat.__main__:main']
    },
    version=version,
    description="TDK Güncel Türkçe Sözlük için komut satırı uygulaması.",
    long_description=long_descr,
    long_description_content_type="text/x-rst",
    keywords="tdk,cli",
    author="Adem Özay",
    author_email="ozayadem@gmail.com",
    url="https://github.com/ademozay/lugat",
    install_requires=['termcolor'],
    python_requries='>=3.7'
)
